Mul

Trait Mul 

1.0.0 (const: unstable) · Source
pub trait Mul<Rhs = Self> {
    type Output;

    // Required method
    fn mul(self, rhs: Rhs) -> Self::Output;
}
The multiplication operator *.

Note that Rhs is Self by default, but this is not mandatory.

Examples

Multipliable rational numbers

use std::ops::Mul;

// By the fundamental theorem of arithmetic, rational numbers in lowest
// terms are unique. So, by keeping `Rational`s in reduced form, we can
// derive `Eq` and `PartialEq`.
#[derive(Debug, Eq, PartialEq)]
struct Rational {
    numerator: usize,
    denominator: usize,
}

impl Rational {
    fn new(numerator: usize, denominator: usize) -> Self {
        if denominator == 0 {
            panic!("Zero is an invalid denominator!");
        }

        // Reduce to lowest terms by dividing by the greatest common
        // divisor.
        let gcd = gcd(numerator, denominator);
        Self {
            numerator: numerator / gcd,
            denominator: denominator / gcd,
        }
    }
}

impl Mul for Rational {
    // The multiplication of rational numbers is a closed operation.
    type Output = Self;

    fn mul(self, rhs: Self) -> Self {
        let numerator = self.numerator * rhs.numerator;
        let denominator = self.denominator * rhs.denominator;
        Self::new(numerator, denominator)
    }
}

// Euclid's two-thousand-year-old algorithm for finding the greatest common
// divisor.
fn gcd(x: usize, y: usize) -> usize {
    let mut x = x;
    let mut y = y;
    while y != 0 {
        let t = y;
        y = x % y;
        x = t;
    }
    x
}

assert_eq!(Rational::new(1, 2), Rational::new(2, 4));
assert_eq!(Rational::new(2, 3) * Rational::new(3, 4),
           Rational::new(1, 2));

Multiplying vectors by scalars as in linear algebra

use std::ops::Mul;

struct Scalar { value: usize }

#[derive(Debug, PartialEq)]
struct Vector { value: Vec<usize> }

impl Mul<Scalar> for Vector {
    type Output = Self;

    fn mul(self, rhs: Scalar) -> Self::Output {
        Self { value: self.value.iter().map(|v| v * rhs.value).collect() }
    }
}

let vector = Vector { value: vec![2, 4, 6] };
let scalar = Scalar { value: 3 };
assert_eq!(vector * scalar, Vector { value: vec![6, 12, 18] });

Required Associated Types

1.0.0 · Source

type Output

The resulting type after applying the * operator.

Required Methods

1.0.0 · Source

fn mul(self, rhs: Rhs) -> Self::Output

Performs the * operation.

Example
assert_eq!(12 * 2, 24);

Implementors

1.0.0 (const: unstable) · Source

impl Mul for f16

1.0.0 (const: unstable) · Source

impl Mul for f32

1.0.0 (const: unstable) · Source

impl Mul for f64

1.0.0 (const: unstable) · Source

impl Mul for f128

1.0.0 (const: unstable) · Source

impl Mul for i8

1.0.0 (const: unstable) · Source

impl Mul for i16

1.0.0 (const: unstable) · Source

impl Mul for i32

1.0.0 (const: unstable) · Source

impl Mul for i64

1.0.0 (const: unstable) · Source

impl Mul for i128

1.0.0 (const: unstable) · Source

impl Mul for isize

1.0.0 (const: unstable) · Source

impl Mul for u8

1.0.0 (const: unstable) · Source

impl Mul for u16

1.0.0 (const: unstable) · Source

impl Mul for u32

1.0.0 (const: unstable) · Source

impl Mul for u64

1.0.0 (const: unstable) · Source

impl Mul for u128

1.0.0 (const: unstable) · Source

impl Mul for usize

1.74.0 (const: unstable) · Source

impl Mul for Saturating<i8>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<i16>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<i32>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<i64>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<i128>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<isize>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<u8>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<u16>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<u32>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<u64>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<u128>

1.74.0 (const: unstable) · Source

impl Mul for Saturating<usize>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<i8>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<i16>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<i32>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<i64>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<i128>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<isize>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<u8>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<u16>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<u32>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<u64>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<u128>

1.0.0 (const: unstable) · Source

impl Mul for core::num::wrapping::Wrapping<usize>

Source

impl Mul for BoxedMontyForm

Source

impl Mul for Limb

1.0.0 (const: unstable) · Source

impl Mul<&f16> for &f16

1.0.0 (const: unstable) · Source

impl Mul<&f16> for f16

1.0.0 (const: unstable) · Source

impl Mul<&f32> for &f32

1.0.0 (const: unstable) · Source

impl Mul<&f32> for f32

1.0.0 (const: unstable) · Source

impl Mul<&f64> for &f64

1.0.0 (const: unstable) · Source

impl Mul<&f64> for f64

1.0.0 (const: unstable) · Source

impl Mul<&f128> for &f128

1.0.0 (const: unstable) · Source

impl Mul<&f128> for f128

1.0.0 (const: unstable) · Source

impl Mul<&i8> for &i8

1.0.0 (const: unstable) · Source

impl Mul<&i8> for i8

1.0.0 (const: unstable) · Source

impl Mul<&i16> for &i16

1.0.0 (const: unstable) · Source

impl Mul<&i16> for i16

1.0.0 (const: unstable) · Source

impl Mul<&i32> for &i32

1.0.0 (const: unstable) · Source

impl Mul<&i32> for i32

1.0.0 (const: unstable) · Source

impl Mul<&i64> for &i64

1.0.0 (const: unstable) · Source

impl Mul<&i64> for i64

1.0.0 (const: unstable) · Source

impl Mul<&i128> for &i128

1.0.0 (const: unstable) · Source

impl Mul<&i128> for i128

1.0.0 (const: unstable) · Source

impl Mul<&isize> for &isize

1.0.0 (const: unstable) · Source

impl Mul<&isize> for isize

1.0.0 (const: unstable) · Source

impl Mul<&u8> for &u8

1.0.0 (const: unstable) · Source

impl Mul<&u8> for u8

1.0.0 (const: unstable) · Source

impl Mul<&u16> for &u16

1.0.0 (const: unstable) · Source

impl Mul<&u16> for u16

1.0.0 (const: unstable) · Source

impl Mul<&u32> for &u32

1.0.0 (const: unstable) · Source

impl Mul<&u32> for u32

1.0.0 (const: unstable) · Source

impl Mul<&u64> for &u64

1.0.0 (const: unstable) · Source

impl Mul<&u64> for u64

1.0.0 (const: unstable) · Source

impl Mul<&u128> for &u128

1.0.0 (const: unstable) · Source

impl Mul<&u128> for u128

1.0.0 (const: unstable) · Source

impl Mul<&usize> for &usize

1.0.0 (const: unstable) · Source

impl Mul<&usize> for usize

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i8>> for &Saturating<i8>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i8>> for Saturating<i8>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i16>> for &Saturating<i16>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i16>> for Saturating<i16>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i32>> for &Saturating<i32>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i32>> for Saturating<i32>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i64>> for &Saturating<i64>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i64>> for Saturating<i64>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i128>> for &Saturating<i128>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<i128>> for Saturating<i128>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<isize>> for &Saturating<isize>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<isize>> for Saturating<isize>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u8>> for &Saturating<u8>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u8>> for Saturating<u8>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u16>> for &Saturating<u16>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u16>> for Saturating<u16>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u32>> for &Saturating<u32>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u32>> for Saturating<u32>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u64>> for &Saturating<u64>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u64>> for Saturating<u64>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u128>> for &Saturating<u128>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<u128>> for Saturating<u128>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<usize>> for &Saturating<usize>

1.74.0 (const: unstable) · Source

impl Mul<&Saturating<usize>> for Saturating<usize>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i8>> for &core::num::wrapping::Wrapping<i8>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i8>> for core::num::wrapping::Wrapping<i8>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i16>> for &core::num::wrapping::Wrapping<i16>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i16>> for core::num::wrapping::Wrapping<i16>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i32>> for &core::num::wrapping::Wrapping<i32>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i32>> for core::num::wrapping::Wrapping<i32>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i64>> for &core::num::wrapping::Wrapping<i64>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i64>> for core::num::wrapping::Wrapping<i64>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i128>> for &core::num::wrapping::Wrapping<i128>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<i128>> for core::num::wrapping::Wrapping<i128>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<isize>> for &core::num::wrapping::Wrapping<isize>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<isize>> for core::num::wrapping::Wrapping<isize>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u8>> for &core::num::wrapping::Wrapping<u8>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u8>> for core::num::wrapping::Wrapping<u8>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u16>> for &core::num::wrapping::Wrapping<u16>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u16>> for core::num::wrapping::Wrapping<u16>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u32>> for &core::num::wrapping::Wrapping<u32>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u32>> for core::num::wrapping::Wrapping<u32>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u64>> for &core::num::wrapping::Wrapping<u64>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u64>> for core::num::wrapping::Wrapping<u64>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u128>> for &core::num::wrapping::Wrapping<u128>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<u128>> for core::num::wrapping::Wrapping<u128>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<usize>> for &core::num::wrapping::Wrapping<usize>

1.14.0 (const: unstable) · Source

impl Mul<&Wrapping<usize>> for core::num::wrapping::Wrapping<usize>

Source

impl Mul<&BoxedMontyForm> for &BoxedMontyForm

Source

impl Mul<&BoxedMontyForm> for BoxedMontyForm

Source

impl Mul<&Limb> for &Limb

Source

impl Mul<&Limb> for Limb

1.0.0 (const: unstable) · Source

impl Mul<f16> for &f16

1.0.0 (const: unstable) · Source

impl Mul<f32> for &f32

1.0.0 (const: unstable) · Source

impl Mul<f64> for &f64

1.0.0 (const: unstable) · Source

impl Mul<f128> for &f128

1.0.0 (const: unstable) · Source

impl Mul<i8> for &i8

1.0.0 (const: unstable) · Source

impl Mul<i16> for &i16

1.0.0 (const: unstable) · Source

impl Mul<i32> for &i32

1.0.0 (const: unstable) · Source

impl Mul<i64> for &i64

1.0.0 (const: unstable) · Source

impl Mul<i128> for &i128

1.0.0 (const: unstable) · Source

impl Mul<isize> for &isize

1.0.0 (const: unstable) · Source

impl Mul<u8> for &u8

1.0.0 (const: unstable) · Source

impl Mul<u16> for &u16

1.0.0 (const: unstable) · Source

impl Mul<u32> for &u32

1.3.0 (const: unstable) · Source

impl Mul<u32> for Duration

1.0.0 (const: unstable) · Source

impl Mul<u64> for &u64

1.0.0 (const: unstable) · Source

impl Mul<u128> for &u128

1.0.0 (const: unstable) · Source

impl Mul<usize> for &usize

1.74.0 (const: unstable) · Source

impl Mul<Saturating<i8>> for &Saturating<i8>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<i16>> for &Saturating<i16>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<i32>> for &Saturating<i32>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<i64>> for &Saturating<i64>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<i128>> for &Saturating<i128>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<isize>> for &Saturating<isize>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<u8>> for &Saturating<u8>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<u16>> for &Saturating<u16>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<u32>> for &Saturating<u32>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<u64>> for &Saturating<u64>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<u128>> for &Saturating<u128>

1.74.0 (const: unstable) · Source

impl Mul<Saturating<usize>> for &Saturating<usize>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<i8>> for &core::num::wrapping::Wrapping<i8>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<i16>> for &core::num::wrapping::Wrapping<i16>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<i32>> for &core::num::wrapping::Wrapping<i32>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<i64>> for &core::num::wrapping::Wrapping<i64>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<i128>> for &core::num::wrapping::Wrapping<i128>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<isize>> for &core::num::wrapping::Wrapping<isize>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<u8>> for &core::num::wrapping::Wrapping<u8>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<u16>> for &core::num::wrapping::Wrapping<u16>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<u32>> for &core::num::wrapping::Wrapping<u32>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<u64>> for &core::num::wrapping::Wrapping<u64>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<u128>> for &core::num::wrapping::Wrapping<u128>

1.14.0 (const: unstable) · Source

impl Mul<Wrapping<usize>> for &core::num::wrapping::Wrapping<usize>

1.31.0 (const: unstable) · Source

impl Mul<Duration> for u32

Source

impl Mul<BoxedMontyForm> for &BoxedMontyForm

Source

impl Mul<Limb> for &Limb

Source

impl<'lhs, 'rhs, T, const N: usize> Mul<&'rhs Simd<T, N>> for &'lhs Simd<T, N>
where T: SimdElement, Simd<T, N>: Mul<Output = Simd<T, N>>,

Source

type Output = Simd<T, N>

Source

impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> Mul for ConstMontyForm<MOD, LIMBS>

Source

type Output = ConstMontyForm<MOD, LIMBS>

Source

impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> Mul<&ConstMontyForm<MOD, LIMBS>> for &ConstMontyForm<MOD, LIMBS>

Source

type Output = ConstMontyForm<MOD, LIMBS>

Source

impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> Mul<&ConstMontyForm<MOD, LIMBS>> for ConstMontyForm<MOD, LIMBS>

Source

type Output = ConstMontyForm<MOD, LIMBS>

Source

impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> Mul<ConstMontyForm<MOD, LIMBS>> for &ConstMontyForm<MOD, LIMBS>

Source

type Output = ConstMontyForm<MOD, LIMBS>

Source

impl<Rhs: AsRef<UintRef>> Mul<Rhs> for &BoxedUint

Source

impl<Rhs: AsRef<UintRef>> Mul<Rhs> for BoxedUint

Source

impl<T> Mul for Checked<T>

Source

impl<T> Mul for NonZero<T>
where T: Mul<T, Output = T>,

Any non-zero integer multiplied by another non-zero integer is definitionally non-zero.

Source

impl<T> Mul for Odd<T>
where T: Mul<T, Output = T>,

Any odd integer multiplied by another odd integer is definitionally odd.

Source

impl<T> Mul<&Checked<T>> for &Checked<T>

Source

impl<T> Mul<&Checked<T>> for Checked<T>

Source

impl<T> Mul<Checked<T>> for &Checked<T>

Source

impl<T, const N: usize> Mul<&Simd<T, N>> for Simd<T, N>
where T: SimdElement, Simd<T, N>: Mul<Output = Simd<T, N>>,

Source

type Output = Simd<T, N>

Source

impl<T, const N: usize> Mul<Simd<T, N>> for &Simd<T, N>
where T: SimdElement, Simd<T, N>: Mul<Output = Simd<T, N>>,

Source

type Output = Simd<T, N>

Source

impl<T: WrappingMul> Mul for crypto_bigint::Wrapping<T>

Source

impl<T: WrappingMul> Mul<&Wrapping<T>> for &crypto_bigint::Wrapping<T>

Source

impl<T: WrappingMul> Mul<&Wrapping<T>> for crypto_bigint::Wrapping<T>

Source

impl<T: WrappingMul> Mul<Wrapping<T>> for &crypto_bigint::Wrapping<T>

Source

impl<const LIMBS: usize> Mul for FixedMontyForm<LIMBS>

Source

impl<const LIMBS: usize> Mul<&FixedMontyForm<LIMBS>> for &FixedMontyForm<LIMBS>

Source

impl<const LIMBS: usize> Mul<&FixedMontyForm<LIMBS>> for FixedMontyForm<LIMBS>

Source

impl<const LIMBS: usize> Mul<FixedMontyForm<LIMBS>> for &FixedMontyForm<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Int<RHS_LIMBS>> for &Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Int<RHS_LIMBS>> for Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Uint<RHS_LIMBS>> for &Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Uint<RHS_LIMBS>> for &Uint<LIMBS>

Source

type Output = Uint<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Uint<RHS_LIMBS>> for Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<&Uint<RHS_LIMBS>> for Uint<LIMBS>

Source

type Output = Uint<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Int<RHS_LIMBS>> for &Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Int<RHS_LIMBS>> for Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Uint<RHS_LIMBS>> for &Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Uint<RHS_LIMBS>> for &Uint<LIMBS>

Source

type Output = Uint<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Uint<RHS_LIMBS>> for Int<LIMBS>

Source

type Output = Int<LIMBS>

Source

impl<const LIMBS: usize, const RHS_LIMBS: usize> Mul<Uint<RHS_LIMBS>> for Uint<LIMBS>

Source

type Output = Uint<LIMBS>

Source

impl<const N: usize> Mul for Simd<f32, N>

Source

impl<const N: usize> Mul for Simd<f64, N>

Source

impl<const N: usize> Mul for Simd<i8, N>
where i8: SimdElement,

Source

impl<const N: usize> Mul for Simd<i16, N>

Source

impl<const N: usize> Mul for Simd<i32, N>

Source

impl<const N: usize> Mul for Simd<i64, N>

Source

impl<const N: usize> Mul for Simd<isize, N>

Source

impl<const N: usize> Mul for Simd<u8, N>
where u8: SimdElement,

Source

impl<const N: usize> Mul for Simd<u16, N>

Source

impl<const N: usize> Mul for Simd<u32, N>

Source

impl<const N: usize> Mul for Simd<u64, N>

Source

impl<const N: usize> Mul for Simd<usize, N>