pub trait Xgcd<Rhs = Self>: Sized {
type Output;
// Required methods
fn xgcd(&self, rhs: &Rhs) -> Self::Output;
fn xgcd_vartime(&self, rhs: &Rhs) -> Self::Output;
}Expand description
Compute the extended greatest common divisor of two integers.
Required Associated Types
Required Methods
Sourcefn xgcd(&self, rhs: &Rhs) -> Self::Output
fn xgcd(&self, rhs: &Rhs) -> Self::Output
Compute the extended greatest common divisor of self and rhs.
Sourcefn xgcd_vartime(&self, rhs: &Rhs) -> Self::Output
fn xgcd_vartime(&self, rhs: &Rhs) -> Self::Output
Compute the extended greatest common divisor of self and rhs in variable time.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.